﻿// For an introduction to the Blank template, see the following documentation:
// http://go.microsoft.com/fwlink/?LinkId=232509
(function () {
    "use strict";

    WinJS.Binding.optimizeBindingReferences = true;

    var app = WinJS.Application;
    var activation = Windows.ApplicationModel.Activation;

    app.onactivated = function (args) {
        if (args.detail.kind === activation.ActivationKind.launch) {
            if (args.detail.previousExecutionState !== activation.ApplicationExecutionState.terminated) {
                // TODO: This application has been newly launched. Initialize
                // your application here.
            } else {
                // TODO: This application has been reactivated from suspension.
                // Restore application state here.
            }

            args.setPromise(WinJS.UI.processAll().then(function completed() 
            {
                displayInit();
                pdfNetInit();
                open_Click();
            }));
        }
    };

    app.oncheckpoint = function (args) {
        // TODO: This application is about to be suspended. Save any state
        // that needs to persist across suspensions here. You might use the
        // WinJS.Application.sessionState object, which is automatically
        // saved and restored across suspension. If you need to complete an
        // asynchronous operation before your application is suspended, call
        // args.setPromise().
    };



    app.start();
    var pdfDraw = pdftron.PDF.PDFDraw();
    var openButton;
    var prevButton;
    var nextButton;
    var isRendering = false;

    var pdfDoc;
    var currentPage;
    var totalPages;

    // touch manipulation
    var target;

    
    function displayInit()
    {
        openButton = document.getElementById("openButton");
        openButton.onclick = open_Click;
        prevButton = document.getElementById("previous");
        prevButton.onclick = prev_Click;
        prevButton.disabled = true;
        nextButton = document.getElementById("next");
        nextButton.onclick = next_Click;
        nextButton.disabled = true;

        target = document.getElementById("DrawCanvas");
        setTarget();

        onResizeCanvas();
        window.addEventListener('resize', onResizeCanvas, false);
        function onResizeCanvas()
        {
            var canDiv = document.getElementById("canvasDiv");
            target.height = canDiv.clientHeight;
            target.width = canDiv.clientWidth;

        }
    }


    function pdfNetInit()
    {
        pdftron.PDFNet.initialize();
    }

    function open_Click()
    {
        // Verify that we are currently not snapped, or that we can unsnap to open the picker
        var currentState = Windows.UI.ViewManagement.ApplicationView.value;
        if (currentState === Windows.UI.ViewManagement.ApplicationViewState.snapped &&
            !Windows.UI.ViewManagement.ApplicationView.tryUnsnap()) {
            // Fail silently if we can't unsnap
            return;
        }

        // Create the picker object and set options
        var openPicker = new Windows.Storage.Pickers.FileOpenPicker();
        openPicker.viewMode = Windows.Storage.Pickers.PickerViewMode.list;
        openPicker.suggestedStartLocation = Windows.Storage.Pickers.PickerLocationId.documentsLibrary;
        // Users expect to have a filtered view of their folders depending on the scenario.
        // For example, when choosing a documents folder, restrict the filetypes to documents for your application.
        openPicker.fileTypeFilter.clear();
        openPicker.fileTypeFilter.append(".pdf");

        // Open the picker for the user to pick a file
        openPicker.pickSingleFileAsync().then(function (file) {
            if (file) {

                file.openAsync(Windows.Storage.FileAccessMode.read).then
                (
                    function (filestream) {
                        pdfDoc = pdftron.PDF.PDFDoc(filestream);

                        isRendering = true;
                        currentPage = 1;
                        totalPages = pdfDoc.getPageCount();

                        goTo(currentPage);
                    }
                );

            } else {
                // file didn't load.
            }
        });
    }


    function prev_Click()
    {
        if (isRendering)
        {
            return;
        }

        isRendering = true;

        currentPage--;

        goTo(currentPage);
    }


    function next_Click() {
        if (isRendering) {
            return;
        }

        isRendering = true;

        currentPage++;

        goTo(currentPage);

    }

    function goTo(pageNumber) {

        if (pageNumber <= 1)
        {
            pageNumber = 1;
            prevButton.disabled = true;
        }
        else
        {
            prevButton.disabled = false;
        }
        if (pageNumber >= totalPages)
        {
            pageNumber = totalPages;
            nextButton.disabled = true;
        }
        else
        {
            nextButton.disabled = false;
        }

        var page = pdfDoc.getPage(pageNumber);

        var width = page.getPageWidth();
        var height = page.getPageHeight();

        var aspectRatio = width / height;

        var wRatio = 1;
        var hRatio = 1;

        var availableWidth = window.innerWidth;
        var availableHeight = window.innerHeight;

        var wRatio = (availableWidth - 210) / width;
        var hRatio = availableHeight /  height;

        var scaleFactor = wRatio;
        if (hRatio < wRatio)
        {
            scaleFactor = hRatio;
        }

        width = width * scaleFactor;
        height = height * scaleFactor;

        pdfDraw.setImageSize(width, height);
        pdfDraw.exportAsStreamAsync(page, "bmp").then(function (imstrm) {

            var url = URL.createObjectURL(imstrm, { oneTimeOnly: true });

            var tag = document.getElementById("docImage");
            tag.src = url;

            isRendering = false;

            // might want to remove the context here instead:
            if (ctx) {
                ctx.clearRect(0, 0, target.width, target.height);
            }

        });

    };

    function setTarget()
    {
        target.pointers = [];
        target.addEventListener("pointerdown", onPointerDown, true);
        //target.addEventListener("pointerover", onPointerOver, true);
        target.addEventListener("pointerup", onPointerUp, true);
        //target.addEventListener("pointerout", onPointerOut, true);
        //target.addEventListener("pointercancel", onPointerCancel, true);
        //target.addEventListener("lostpointercapture", onLostPointerCapture, true);
        target.addEventListener("pointermove", onPointerMove, true);
        //target.addEventListener("wheel", onMouseWheel, false);

        target.ondragstart = function () { return false; };
    };


    var pointerDown = false;
    var mainPointer = -1;
    var ctx;

    var startpoint;
    var imgstart;
    var endpoint;
    var imgend;

    function onPointerDown(e)
    {
        e.cancelBubble = true;

        if (Object.keys(target.pointers).length > 0)
        {
            mainPointer = -1;
            // canels stuff
            return;
        }

        target.pointers[e.pointerId] = e.pointerType;


        var imgelem = document.getElementById("docImage");

        var ptimg = e.getCurrentPoint(document.getElementById("docImage"));
        var ptcan = e.getCurrentPoint(document.getElementById("DrawCanvas"));

        if (ptimg.position.x < 0 || ptimg.position.x > imgelem.width )
        {
            return;
        }
        // dop for height

        var dc = document.getElementById("DrawCanvas");
        ctx = dc.getContext("2d");

        startpoint = ptcan.position;
        imgstart = ptimg.position;

        mainPointer = e.pointerId;

    };

    function onPointerMove(e)
    {
        e.cancelBubble = true;
        if (mainPointer === -1)
        {
            return;
        }

        var ptimg = e.getCurrentPoint(document.getElementById("docImage"));
        var ptcan = e.getCurrentPoint(document.getElementById("DrawCanvas"));

        // check bound here

        endpoint = ptcan.position;
        imgend = ptimg.position;

        ctx.clearRect(0, 0, target.width, target.height);
        ctx.beginPath();
        ctx.moveTo(startpoint.x, startpoint.y);
        ctx.lineTo(endpoint.x, endpoint.y);

        ctx.strokeStyle = '#00ff00';
        ctx.stroke();
    };

    function onPointerUp(e)
    {
        e.cancelBubble = true;

        if (mainPointer !== -1)
        {
            mainPointer = -1;

            var img = document.getElementById("docImage");

            // get the positions as percentages of the image
            var spX = imgstart.x / img.width;
            var spY = imgstart.y / img.height;

            var epX = imgend.x / img.width;
            var epY = imgend.y / img.height;

            // translate these intp page coordinates.
            var pdfpage = pdfDoc.getPage(currentPage);
            var pageRect = pdfpage.getCropBox();

            var startX = spX * pageRect.width();
            var endX = epX * pageRect.width();

            var startY = (1 - spY) * pageRect.height();
            var endY = (1 - epY) * pageRect.height();



            var rect = pdftron.PDF.Rect(Math.min(startX, endX), Math.min(startY, endY), Math.max(startX, endX), Math.max(startY, endY));

            var SDFDoc = pdfDoc.getSDFDoc();
            var annot = pdftron.PDF.Annots.Line.create(SDFDoc, rect);
            annot.setStartPoint(pdftron.PDF.Point(startX, startY));
            annot.setEndPoint(pdftron.PDF.Point(endX, endY));
            annot.setColor(pdftron.PDF.ColorPt(0, 1, 0));
            annot.setOpacity(1);
            annot.refreshAppearance();
            pdfpage.annotPushBack(annot);

            goTo(currentPage);
        }
        if (target.pointers[e.pointerId])
        {
            delete target.pointers[e.pointerId];
        }
        
    };

})();
